package cn.kdan.cloud.pdf.office.payment.listener;

import cn.kdan.cloud.pdf.office.api.account.dto.UpdateUserForOrderDTO;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.api.product.enums.PlatformEnum;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserStatusEnum;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.payment.service.OrdersService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import cn.kdan.cloud.pdf.office.api.account.utils.SubscriptionUtil;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.rabbitmq.client.Channel;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/1/11
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class PaymentListener {

    private final OrdersService ordersService;

    private final SubscriptionsService subscriptionsService;

    private final UserApi userApi;

    /**
     * 关闭订单 监听
     *
     * @param message message
     * @param channel channel
     * @param orderId orderId
     */
    @RabbitListener(queues = RabbitMqConstant.PAYMENT_ORDER_DELAY_QUEUE)
    public void paymentOrderDelayQueue(Message message, Channel channel, String orderId) {
        try {
            log.info("关闭订单 监听内容：{}", orderId);
            ordersService.closeOrder(orderId);
            log.info("订单关闭成功 :{}", orderId);
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            log.error("消息消费失败:" + e.getMessage(), e);
        }
    }

    /**
     * 订阅过期 监听
     *
     * @param message        message
     * @param channel        channel
     * @param subscriptionId subscriptionId
     */
    @RabbitListener(queues = RabbitMqConstant.PAYMENT_SUBSCRIPTION_EXPIRED_DELAY_QUEUE)
    public void paymentSubscriptionExpiredDelayQueue(Message message, Channel channel, String subscriptionId) {
        try {
            log.info("订阅过期 监听内容：{}", subscriptionId);
            // 判断当前用户是否有有效的订阅模式
            SubscriptionsVO subscriptionsVO = subscriptionsService.getSubscriptionBySubscriptionId(subscriptionId);
            SubscriptionsVO subscription = subscriptionsService.getSubscriptions(subscriptionsVO.getUserId(), PlatformEnum.getEnumByValue(subscriptionsVO.getPlatform()));
            LocalDateTime now = LocalDateTime.now();
            LocalDateTime endDate = MyDateUtils.dateToLocalDateTime(subscription.getEndDate());
            if (now.compareTo(endDate) > 0) {
                // 修改用户状态
                // 获取当前用户的订阅状态
                List<UserSubscriptionInfoVO> subscriptionInfoList = userApi.getInfoById(subscription.getUserId()).getResult().getSubscriptionInfoList();
                UserSubscriptionInfoVO userSubscriptionInfoVO = SubscriptionUtil.getUserSubscriptionInfoByPlatform(subscriptionInfoList, subscription.getPlatform());
                Integer userStatus = Integer.MIN_VALUE;
                if (!ObjectUtils.isEmpty(userSubscriptionInfoVO)) {
                    userStatus = userSubscriptionInfoVO.getStatus();
                    PDFOfficeUserStatusEnum userStatusEnum = PDFOfficeUserStatusEnum.getEnumByValue(userSubscriptionInfoVO.getStatus());
                    if (userStatusEnum == PDFOfficeUserStatusEnum.SUBSCRIPTION_IN_PROGRESS){
                        userStatus = PDFOfficeUserStatusEnum.SUBSCRIPTION_EXPIRED.value();
                    }else if (userStatusEnum == PDFOfficeUserStatusEnum.TRIAL){
                        userStatus = PDFOfficeUserStatusEnum.TRIAL_EXPIRED.value();
                    }
                }
                // 修改用户状态
                UpdateUserForOrderDTO paySuccessDTO = new UpdateUserForOrderDTO();
                paySuccessDTO.setUserId(subscription.getUserId());
                paySuccessDTO.setPlatform(subscription.getPlatform());
                paySuccessDTO.setUserStatus(userStatus);
                userApi.updateUserForOrder(paySuccessDTO);
            }
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
            log.info("订阅过期 处理成功 :{}", subscriptionId);
        } catch (Exception e) {
            log.error("消息消费失败:" + e.getMessage(), e);
        }
    }


}
