package cn.kdan.cloud.pdf.office.sso.controller;

import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.common.dto.UserResetPasswordDTO;
import cn.kdan.cloud.pdf.office.sso.constant.AuthConstant;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.sso.enums.VerifyTypeEnum;
import cn.kdan.cloud.pdf.office.sso.service.AuthService;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.vo.TokenVO;
import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotBlank;
import java.security.Principal;
import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("/auth")
public class AuthController {

    @Autowired
    private EmailApi emailApi;

    private final AuthService authService;

    @Autowired
    public AuthController(AuthService authService) {
        this.authService = authService;
    }


    /**
     * @param clientId
     * @param principal
     * @param request
     * @Description: 获取授权码，登陆时调用
     * @Return: com.wondersgroup.cloud.cmp.common.pojo.ResultMap<java.lang.String>
     * @Date: 2021/2/10 10:31
     */
    @GetMapping("/getAuthorizeCode")
    public ResultMap<String> authorize(@RequestParam String clientId, Principal principal, HttpServletRequest request) {
        String code = authService.getAuthorizeCode(getUserId(principal), clientId, request);
        if (StringUtils.isNotEmpty(code)) {
            return new ResultMap<>(CommonConstant.SUCCESS, AuthConstant.GET_AUTHORIZE_CODE_SUCCESS, code);
        } else {
            return new ResultMap<>(CommonConstant.ERROR, AuthConstant.EXCEPTION_MSG_GET_CODE_FAIL);
        }
    }

    /**
     * @param clientId
     * @param code
     * @param request
     * @Description: 获取token  登陆时调用
     * @Date: 2021/2/10 10:32
     */
    @GetMapping("/getToken")
    public ResultMap<TokenVO> getToken(@RequestParam String clientId, @RequestParam String code, HttpServletRequest request) {
        TokenVO tokenVO = authService.getToken(code, clientId, request);
        if (tokenVO == null) {
            return new ResultMap<>(AuthConstant.EXCEPTION_CODE_GET_TOKEN_FAIL, AuthConstant.EXCEPTION_MSG_GET_TOKEN_FAIL);
        } else {
            return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, tokenVO);
        }
    }


    /**
     * 邮箱验证码登录
     * @param email 邮箱
     * @param password 验证码1
     * @param appId appid
     * @param platformType 平台类型 1 官网 2 后台
     * @param request
     * @return
     */
    @PostMapping("/emailLogin")
    public ResultMap<TokenVO> emailLogin(@RequestParam String email, @RequestParam String password, @RequestParam String appId,
                                       @RequestParam String platformType,  @RequestParam String deviceSign,HttpServletRequest request) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, authService.emailLogin(email,password,appId,platformType,deviceSign,request));
    }

    /**
     * 邮箱注册
     *
     * @param userRegisterDTO dto
     * @return TokenVO
     */
    @PostMapping("/emailRegister")
    public ResultMap<TokenVO> emailRegister(@RequestBody UserRegisterDTO userRegisterDTO) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, authService.emailRegister(userRegisterDTO));
    }

    @PostMapping("/resetPassword")
    public ResultMap<Boolean> resetPassword(@RequestBody UserResetPasswordDTO userResetPasswordDTO) {
        authService.resetPassword(userResetPasswordDTO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE );
    }

    /**
     * 验证用户是否存在，邮箱格式是否正确
     *
     * @param userRegisterDTO userRegisterDTO
     * @return
     */
    @PostMapping("/validUserRegisterParam")
    public ResultMap<Boolean> validUserRegisterParam(@RequestBody UserRegisterDTO userRegisterDTO) {
        authService.validUserRegisterParam(userRegisterDTO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }

    /**
     * 验证登录用户是否存在，邮箱格式是否正确
     *
     * @param userRegisterDTO userRegisterDTO
     * @return
     */
    @PostMapping("/validUserLoginParam")
    public ResultMap<Boolean> validUserLoginParam(@RequestBody UserRegisterDTO userRegisterDTO) {
        authService.validUserLoginParam(userRegisterDTO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }


    @GetMapping("/getVerifyCode")
    public ResultMap<Boolean> getVerifyCode(@RequestParam String action,@RequestParam Integer type,@NotBlank(message = "接收者不能为空") @RequestParam String receiver,@RequestParam String appId) {
        EmailCodeTypeEnum actionEnum = EmailCodeTypeEnum.getEnumByValue(action);
        if (actionEnum == null) {
            return new ResultMap<>(CommonConstant.ERROR, AuthConstant.EXCEPTION_MSG_CODE_ACTION_ERROR, false);
        }
        VerifyTypeEnum verifyTypeEnum = VerifyTypeEnum.getEnumByCode(type);
        if (verifyTypeEnum == null) {
            return new ResultMap<>(CommonConstant.ERROR, AuthConstant.EXCEPTION_MSG_CODE_TYPE_ERROR, false);
        }
        if (authService.getVerifyCode(actionEnum, verifyTypeEnum, receiver,appId)) {
            return new ResultMap<>(CommonConstant.SUCCESS, VerifyTypeEnum.PHONE.code().equals(type) ?AuthConstant.SUCCESS_MSG_CODE_SEND:AuthConstant.SUCCESS_MSG_CODE_SEND_MAIL, true);
        } else {
            return new ResultMap<>(CommonConstant.ERROR, VerifyTypeEnum.PHONE.code().equals(type) ?AuthConstant.EXCEPTION_MSG_CODE_SEND_ERROR:AuthConstant.EXCEPTION_MAIL_CODE_SEND_ERROR, false);
        }
    }


    /**
     * 实时检查邮件验证码
     * @param type 验证码类型  login  user_register
     * @param account 账号
     * @param code 验证码
     * @return
     */
    @GetMapping("/isEmailCodeValid")
    public ResultMap<Boolean> isEmailCodeValid(@RequestParam String type, @RequestParam String account, @RequestParam String code,@RequestParam String appId) {
        EmailCodeTypeEnum typeEnum = EmailCodeTypeEnum.getEnumByValue(type);
        authService.isEmailCodeValid(typeEnum,account,code,appId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }


    private String getUserId(Principal principal) {
        if (principal instanceof UsernamePasswordAuthenticationToken) {
            return ((CustomUserDetails) (((UsernamePasswordAuthenticationToken) principal).getPrincipal())).getId();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }


}
